<?php

/**
 * Class Users
 *
 * Please note:
 * Don't use the same name for class and method, as this might trigger an (unintended) __construct of the class.
 * This is really weird behaviour, but documented here: http://php.net/manual/en/language.oop5.decon.php
 *
 */
class Users extends Controller
{
    /**
     * PAGE: index
     * This method handles user's default page
     */
    public function index()
    {
        session_start();
        if (isset($_SESSION['valid'])) {
            $title = 'Users';
            require_once 'application/views/_templates/header.php';
            require_once 'application/views/_templates/sidemenu.php';
            require_once 'application/views/_templates/topnavigation.php';
            require_once 'application/views/users/index.php';
            require_once 'application/views/_templates/footer.php';
        }
        else {
            header('Location: ' . URL . 'login');
        }
    }

    /** 
     * PAGE: list (API)
     * This method returns all users in json format
    */
    public function listUsers()
    {
        if ($_SERVER['REQUEST_METHOD'] == 'GET')
        {
            try {
                $users_model = $this->loadModel('UsersModel');
                $users = $users_model->listUsers();

                header("Access-Control-Allow-Origin: *");
                header("Content-Type: application/json; charset=UTF-8");
                echo $users;
            } catch (Throwable $e) {
                header($_SERVER['SERVER_PROTOCOL'] . ' 500 Internal Server Error', true, 500);
                echo json_encode(array('error' => $e->getMessage()));
            }
        }
        else
        {
            header($_SERVER['SERVER_PROTOCOL'] . ' 400 Bad Request', true, 400);
            exit;
        }
    }

    /** 
     * PAGE: add (API)
     * This method add's a new user
    */
    public function add()
    {
        if ($_SERVER['REQUEST_METHOD'] == 'POST')
        {
            try {
                if (isset($_POST["username"]) && !empty($_POST["username"])) { 
                    $users_model = $this->loadModel('UsersModel');
                    $users_model->addUser($_POST["username"], $_POST["password"], $_POST["confirm_password"], $_POST["user_type"]);
                }
                else{
                    header($_SERVER['SERVER_PROTOCOL'] . ' 500 Internal Server Error', true, 500);
                    echo json_encode(array('error' => 'Invalid request.'));
                }
            } catch (Throwable $e) {
                header($_SERVER['SERVER_PROTOCOL'] . ' 500 Internal Server Error', true, 500);
                echo json_encode(array('error' => $e->getMessage()));
            }
        }
        else
        {
            header($_SERVER['SERVER_PROTOCOL'] . ' 400 Bad Request', true, 400);
            exit;
        }
    }

    /** 
     * PAGE: reset (API)
     * This method reset's user password
    */
    public function reset()
    {
        if ($_SERVER['REQUEST_METHOD'] == 'POST')
        {
            try {
                if (isset($_POST["id"]) && !empty($_POST["id"])) { 
                    $users_model = $this->loadModel('UsersModel');
                    $users_model->resetPassword($_POST["id"], $_POST["password"], $_POST["confirm_password"]);
                }
                else{
                    header($_SERVER['SERVER_PROTOCOL'] . ' 500 Internal Server Error', true, 500);
                    echo json_encode(array('error' => 'Invalid request.'));
                }
            } catch (Throwable $e) {
                header($_SERVER['SERVER_PROTOCOL'] . ' 500 Internal Server Error', true, 500);
                echo json_encode(array('error' => $e->getMessage()));
            }
        }
        else
        {
            header($_SERVER['SERVER_PROTOCOL'] . ' 400 Bad Request', true, 400);
            exit;
        }
    }

    /** 
     * PAGE: delete (API)
     * This method delete's a user
    */
    public function delete()
    {
        if ($_SERVER['REQUEST_METHOD'] == 'POST') {
            try {
                if (isset($_POST["id"]) && !empty($_POST["id"])) { 
                    $users_model = $this->loadModel('UsersModel');
                    $users_model->deleteUser($_POST["id"]);
                }
                else{
                    header($_SERVER['SERVER_PROTOCOL'] . ' 500 Internal Server Error', true, 500);
                    echo json_encode(array('error' => 'Invalid request.'));
                }
            } catch (Throwable $e) {
                header($_SERVER['SERVER_PROTOCOL'] . ' 500 Internal Server Error', true, 500);
                echo json_encode(array('error' => $e->getMessage()));
            }
        }
        else
        {
            header($_SERVER['SERVER_PROTOCOL'] . ' 400 Bad Request', true, 400);
            exit;
        }
    }
}
