<?php

class LoginModel
{
    /**
     * Every model needs a database connection, passed to the model
     * @param object $db A PDO database connection
     */
    function __construct($db) {
        try {
            $this->db = $db;
        } catch (PDOException $e) {
            exit('Database connection could not be established.');
        }
    }

    /**
     * Authenticate User
     * @param string $username username
     * @param string $password password
     */
    public function authenticateUser($username, $password)
    {
        try {
            require_once('application/libs/utilities.php');

            // Clean the input from javascript
            $username = str_replace(' ', '', $username);
            $username = preg_replace('/\s+/', '', $username);
            $username = Utilities::cleanInput($username);

            $password = Utilities::cleanInput($password);
            //END Clean the input from javascript

            $sql = "SELECT password, user_type FROM user WHERE username = :username  
                    AND (user_type = 'super_admin' OR user_type = 'administrator')";
            
            $query = $this->db->prepare($sql);
            $query->execute(array(':username' => $username));

            if (!$query->rowCount())
            {
                throw new Exception('USER_ERR-207: Invalid username or password.');
            }
            
            $row = $query->fetch(PDO::FETCH_ASSOC);
            $passwordHash = $row["password"];
            $userType = $row["user_type"];

            $verify = password_verify($password, $passwordHash);

            if($verify)
            {
                //Login Success
                //TODO Set Sesssion and Redirect Page
                Utilities::setSession($username, $userType);
                echo 'valid';
            }
            else
            {
                Utilities::clearSession();
                throw new Exception('USER_ERR-207: Invalid username or password.');
            }

            //$password = passwordHash($password, PASSWORD_DEFAULT);

        } catch (Throwable $e) {
            throw new Exception($e);
        }
    }

    public function logout()
    {
        try {
            require_once('application/libs/utilities.php');
            Utilities::clearSession();
            echo 'valid';
        } catch (Throwable $e) {
            throw new Exception($e);
        }
    }
}
