(function ($) {

    var userTable;
    var baseUrl;

    $.users = {
        init: function(url) {
            baseUrl = url;

            /*Assign Control Events*/
            $.users.addEventHandler($('.btn-add'));

            /* Initialize users table */
            $.users.initTable();

            /* Initialize validation */
            $.users.initValidation();

            /* Fix event handling after search */
            $('#userDT').on('draw.dt', function () {
                $.users.resetEventHandler($('.btn-edit-record'));
                $.users.deleteEventHandler($('.btn-delete-record'));
            });
        },
        initTable: function() {

            userTable = $('#userDT').DataTable( {
                'lengthChange': false,
                'processing': true,
                'serverSide': true,
                "order": [[ 1, "asc" ]],
                'ajax': {
                    'url' : baseUrl + 'users/listUsers'
                },
                'columnDefs': [
                    {
                        "targets": [0],
                        "visible": false,
                        "searchable": false
                    },
                    { 
                        "render": actionlinks,
                        "data": null,         
                        "targets": [3], 
                        "width": "15%", 
                        "targets": 3,
                        "searchable": false,
                        "sortable": false
                    },
                    { 
                        "targets" : [2],
                        "render" : function (data, type, row) {
                           if(data=='standard_user') {
                                return 'Standard User';
                           }
                           else if (data=='administrator') {
                               return 'Administrator';
                           }
                           else {
                               return '';
                           }
                        }
                    }
                ],
                /* Table initialization complete */
                "initComplete": function(settings, json) {
                    $.users.resetEventHandler($('.btn-edit-record'));
                    $.users.deleteEventHandler($('.btn-delete-record'));
                }
            });

            function actionlinks(data, type, full) {
                return '<button title="Reset Password" class="btn btn-warning btn-xs btn-edit-record" data-id=' + full[0] + '><i class="fa fa-undo"></i></button>&nbsp;&nbsp;<button title="Delete" class="btn btn-danger btn-xs btn-delete-record" data-id=' + full[0] + '><i class="fa fa-trash"></i></button>';
            }

        },
        initValidation: function() {
            
            jQuery.validator.addMethod("noSpace", function(value, element) { 
                return value.indexOf(" ") < 0 && value != ""; 
            }, "No space please and don't leave it empty");

            var userForm = $("#user_form").validate({
                rules: {
                    username: {
                        required: true,
                        noSpace: true,
                        minlength: 6
                    },
                    password: {
                        required: true,
                        minlength: 5
                    },
                    confirm_password: {
                        required: true,
                        minlength: 5,
                        equalTo: "#password"
                    },
                },
                messages: {
                    username: {
                        required: "Please enter a username",
                        minlength: "Your username must consist of at least 6 characters"
                    },
                    password: {
                        required: "Please provide a password",
                        minlength: "Your password must be at least 5 characters long"
                    },
                    confirm_password: {
                        required: "Please provide a password",
                        minlength: "Your password must be at least 5 characters long",
                        equalTo: "Please enter the same password as above"
                    }
                },
                submitHandler: function () {
                    var data = $.users.constructSaveModel();
                    $.users.save(data);
                }
            });
    
            var resetForm = $("#reset_form").validate({
                rules: {
                    password: {
                        required: true,
                        minlength: 5
                    },
                    confirm_password: {
                        required: true,
                        minlength: 5,
                        equalTo: "#password"
                    },
                },
                messages: {
                    password: {
                        required: "Please provide a password",
                        minlength: "Your password must be at least 5 characters long"
                    },
                    confirm_password: {
                        required: "Please provide a password",
                        minlength: "Your password must be at least 5 characters long",
                        equalTo: "Please enter the same password as above"
                    }
                },
                submitHandler: function () {
                    var data = $.users.constructResetModel();
                    $.users.resetPassword(data);
                }
            });
        },
        constructSaveModel: function() {
            var data = {
                'username': $('#username').val(),
                'password': $('#password').val(),
                'confirm_password': $('#confirm_password').val(),
                'user_type': $('#user_type').val()
            };

            return data;
        },
        constructResetModel: function() {
            var data = {
                'id': $('#user_id').val(),
                'password': $('#newPassword').val(),
                'confirm_password': $('#confirm_newPassword').val()                
            };

            return data;
        },
        constructDeleteModel: function(id) {
            var data = {
                'id': id
            };

            return data;
        },
        addEventHandler: function(e) {
            $(e).off('click').on('click', function(e){
                $('#user_form')[0].reset();
                $("label.error").hide();
                $(".error").removeClass("error");
                $('#userModal').modal('show');
            })
        },
        resetEventHandler: function(e) {
            $(e).off('click').on('click', function(element){
                $('#reset_form')[0].reset();
                $("label.error").hide();
                $(".error").removeClass("error");
                $('#resetModal').modal('show');
                $('#user_id').val(element.currentTarget.dataset.id); 
            })
        },
        deleteEventHandler: function(e) {
            $(e).off('click').on('click', function(element){
                swal({
                    title: "Confirmation",
                    text: "Are you sure you want to delete user?",
                    icon: "warning",
                    buttons: true,
                    dangerMode: true,
                })
                .then((willDelete) => {
                    if (willDelete) {
                        var data = $.users.constructDeleteModel(element.currentTarget.dataset.id); 
                        $.users.delete(data);
                    }
                });
            })
        },
        save: function(data) {

            $('#userModal').modal('hide');

            $.ajax({
                url: baseUrl + "users/add",
                type: "post",
                data: data,
                success: function (response) {
                    swal("Great", "User saved successfully.", "success"); 
                    userTable.ajax.reload(function(){
                        //Reasign Action Button Event Handler on Reload
                        $.users.resetEventHandler($('.btn-edit-record'));
                        $.users.deleteEventHandler($('.btn-delete-record'));
                    });                
                },
                error: function(jqXHR, textStatus, errorThrown) {
                    var response = jQuery.parseJSON(jqXHR.responseText);

                    if(response != null)
                    {
                        var n = response.error.indexOf("USER_ERR-200");
                        var isError = false;

                        if (n > -1)
                        {
                            isError = true;
                            swal("Oops...", "Username already exist.", "error");
                        }

                        if(!isError) {
                            swal("Oops...", "Something went wrong!", "error");
                        }
                    }
                    else{
                        swal("Oops...", "Something went wrong!", "error");
                    }
                }
            });
        },
        resetPassword: function(data){
            $('#resetModal').modal('hide');

            $.ajax({
                url: baseUrl + "users/reset",
                type: "post",
                data: data,
                success: function (response) {
                    swal("Great", "Password reset successful.", "success"); 
                    userTable.ajax.reload(function(){
                        //Reasign Action Button Event Handler on Reload
                        $.users.resetEventHandler($('.btn-edit-record'));
                        $.users.deleteEventHandler($('.btn-delete-record'));
                    });                
                },
                error: function(jqXHR, textStatus, errorThrown) {
                    swal("Oops...", "Something went wrong!", "error");
                }
            });
        },
        delete: function(data) {
            $.ajax({
                url: baseUrl + "users/delete",
                type: "post",
                data: data,
                success: function (response) {
                    swal("Great", "User deleted successfully.", "success"); 
                    userTable.ajax.reload(function(){
                        //Reasign Action Button Event Handler on Reload
                        $.users.resetEventHandler($('.btn-edit-record'));
                        $.users.deleteEventHandler($('.btn-delete-record'));
                    });                
                },
                error: function(jqXHR, textStatus, errorThrown) {
                    swal("Oops...", "Something went wrong!", "error");
                }
            });
        }
    };

})(window.jQuery);